-- stylua: ignore start
import 'CoreLibs/animation.lua'
import 'CoreLibs/animator.lua'
import 'CoreLibs/easing.lua'
import 'CoreLibs/graphics.lua'
import 'CoreLibs/object.lua'
import 'CoreLibs/timer.lua'
import 'CoreLibs/ui.lua'
import 'CoreLibs/utilities/where.lua'
-- stylua: ignore end

import("../lib/tiny.lua")
import("tiny-tools.lua")
import("systems/filter-types.lua")
import("systems/collision-detection.lua")
import("systems/draw.lua")
import("systems/gravity.lua")
import("systems/velocity.lua")
import("booster.lua")
import("burger.lua")

local tiny <const> = tiny
local gfx <const> = playdate.graphics
playdate.display.setRefreshRate(50)
gfx.setBackgroundColor(gfx.kColorWhite)

burger = Burger.new()
floor = {
    position = { x = 0, y = 230 },
    size = { x = 10000, y = 10 },
    isSolid = true,
}
function floor:elasticity(velocity)
    velocity.x = velocity.x * 0.9
    velocity.y = velocity.y * -0.7
end
function floor:draw()
    gfx.fillRect(floor.position.x, floor.position.y, floor.size.x, floor.size.y)
end

Camera = {
    pan = {
        x = 0,
        y = 0,
    },
}

world = tiny.world(fallSystem, velocitySystem, collisionDetection, drawSystem, burger, floor)
for i = 1, Booster.cacheSize() do
    world:addEntity(Booster.get(i * 60 + (math.random(0, 50)), math.random(50, 200)))
end

function playdate.update()
    local deltaSeconds = playdate.getElapsedTime()
    playdate.resetElapsedTime()
    gfx.clear(gfx.kColorWhite)
    playdate.drawFPS(5, 5)

    Camera.pan.x = math.min(0, -burger.position.x + 200)
    Camera.pan.y = burger.position.y + 120
    local newestX = Booster.newestBooster().position.x
    local panX = -Camera.pan.x
    printTable({ panX = panX, newestX = newestX })
    local offset = 600
    if newestX < panX then
        Booster.get(panX + offset + (math.random(0, 50)), math.random(50, 200))
    end
    floor.position.x = -Camera.pan.x - offset
    gfx.setDrawOffset(Camera.pan.x, Camera.pan.y)

    world:update(deltaSeconds)

    if playdate.buttonJustPressed(playdate.kButtonA) then
        Burger.reset(burger)
        for i = 1, Booster.cacheSize() do
            Booster.get(i * 60 + (math.random(0, 50)), math.random(50, 200))
        end
    end
end
